<?php

namespace Vialibri\HarvestFormatting;

class DataValidator {

  public function validatedIds($ids) {
    if(!is_array($ids)) {
      throw new FormatException('The value passed as the ID list is not an array.');
    } else {
      foreach ($ids as $i => $id) {
        if(!is_numeric($id) && !is_string($id)) {
          throw new FormatException("The value at index $i in the ID array is not a number or a string.");
        } elseif($id === '') {
          throw new FormatException("The value at index $i in the ID array is an empty string.");
        }
      }
    }

    return $ids;
  }

  public function validatedBookData($ids, $bookData) {
    $validated = array();

    if(!is_array($ids)) {
      throw new FormatException('The value passed as the ID list is not an array.');
    } elseif(!is_array($bookData)) {
      throw new FormatException('The value passed as the book list is not an array.');
    } else {
      foreach ($bookData as $book) {
        if(!is_array($book)) {
          throw new FormatException("The value at index $i in the book array is not an array.");
        }

        $validated[] = $this->validatedBook($ids, $book);
      }
    }

    return $validated;
  }

  private function validatedBook(array $ids, array $book) {
    $errors = array();

    if(isset($book['source_id']) && !in_array($book['source_id'], $ids)) {
      $errors[] = "The source_id {$book['source_id']} is not present in the ID array.";
    }

    foreach (array_keys(FieldDetails::$fields) as $key) {
      $errors = array_merge($errors, $this->errorsForField($book, $key));
    }

    $superfluousKeys = array_diff(array_keys($book), array_keys(FieldDetails::$fields));
    if(count($superfluousKeys) > 0) {
      $errors[] = 'The following values have been set when they should not be: '
                    .implode(', ', $superfluousKeys);
    }

    if(count($errors) > 0) {
      $book['errors'] = $errors;
    }

    return $book;
  }

  private function errorsForField(array $book, $key) {
    $errors = array();

    $fieldInfo = FieldDetails::$fields[$key];

    if($fieldInfo['required'] && empty($book[$key])) {
      $errors[] = "Required value $key is missing (or it's set to null, 0 or an empty string).";
    }

    if(isset($book[$key]) && $book[$key] !== '') {
      $value = $book[$key];

      switch ($fieldInfo['type']) {
        case 'boolean':
          if(!is_bool($value)) {
            $errors[] = "$key should be a boolean value.";
          }
          break;

        case 'date':
          if(!is_string($value) || !preg_match('/^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$/', $value)) {
            $errors[] = "$key should be a string in the format Y-m-d H:i:s, e.g. 2016-12-16 13:03:12.";
          }
          break;

        case 'delimited':
          if(is_array($value)) {
            foreach ($value as $keyword) {
              if(!is_string($keyword)) {
                $errors[] = "All values in the $key array should be strings. $keyword is not a string.";
                break;
              }
            }
          } elseif(!is_string($value)) {
            $errors[] = "$key should be a string or an array.";
          }
          break;

        case 'price':
          if(is_string($value) && !preg_match('/^\d*(\.\d\d)?$/', $value)) {
            $errors[] = "$key should either be in the format 123 or 123.45. It should not include a currency symbol.";
          } else if(!is_numeric($value)) {
            $errors[] = "$key should either be a number or a string.";
          } else if($value <= 0) {
            $errors[] = "$key should be positive.";
          }
          break;

        case 'url':
          if(substr(strtolower($value), 0, 4) !== 'http') {
            $errors[] = "$key doesn't look like a valid URL. It doesn't start with \"http\".";
          }
          break;
      }
    }

    return $errors;
  }

}
